/*
 
  LANChat Linux version 1.0.2 fully compatible with windows LC40
  Based on LC 4.0 specification by Piotr Kowalski <goro@vc.pl> 
  
  (c) Mateusz Skoczewski <melun@friko2.onet.pl>
  Antoby <antoby@mail.com>
 
  ----------------------------------------------------------------------
  The Linux LANChat and related utilities are free software; you can
  redistribute it and/or modify it under the terms of the GNU Library
  general Public License as published by the Free Software Foundation;
  either version 2 of the License, or (at your option) any later version.
 
*/

#include "lanchat.h"
#include <curses.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netinet/in.h>

WINDOW *inputwin, *defwin, *headwin, *split1win, *split2win, *split3win, *userwin;

char host[ 30 ], logname[ 20 ], myip[ 16 ], since[ 9 ], away[ 250 ],
     topic[ 75 ], topictime[ 9 ], readbuffer[ 4096 ];

unsigned int  send_size, pop_mode;
unsigned long bytes_rcv=0, total_msg=0;

FILE* f;

/********************************* main *****************************************/

int main( int argc, char **argv )
{
    int ssock, rsock;
    int buf_size;
    struct sockaddr_in toaddr, outaddr, oth;
    pid_t cp, cp2;
    int i;
    int recv, send;
    bool finish, option, gethost=1, usenick=0;
    int away_mode = 0;
    unsigned char addr;

    char chr;
    extern char *optarg;         /* getopt variable declarations */
    extern int optind;
    extern int optopt;
    extern int opterr;

    register struct hostent *hostptr;
    struct in_addr *ptr;

    while ((chr = getopt(argc, argv, "h:")) != EOF)
    switch(chr){
	case 'h':
	    gethost=0;
	    strncpy(host, optarg, 30);
	    host[29]='\0';
	    break;
	default:
	    usage();
    };
    argc -= optind;
    argv += optind;

    if(argc==1){
        strncpy( logname, argv[0], 19 );
	logname[19]='\0';
    }else if(argc==0){
        strncpy( logname, getenv( "LOGNAME" ), 19);
	logname[19]='\0';
    } else
	usage();

    hsnd.cCrypt = CRYPT_DEFAULT;

    hsnd.cHead[ 0 ] = 'L';
    hsnd.cHead[ 1 ] = 'C';
    hsnd.cHead[ 2 ] = '4';
    hsnd.cHead[ 3 ] = '0';

    hsnd.nR = COLOR_R;
    hsnd.nG = COLOR_G;
    hsnd.nB = COLOR_B;

    /* INITIALIZATION STUFF */
    /************************/

    umask( 0 );

    printf( "Initializing LANChat...\n" );

    if(gethost)
	gethostname( host, 30 );
    hostptr = gethostbyname( host );
    ptr = ( struct in_addr* ) * hostptr->h_addr_list;

    strncpy( host, hostptr->h_name, 29 ); host[29]='\0';
    strcpy( myip, inet_ntoa( *ptr ));  /* safe - 255.255.255.255 max 16 */

    addr = *( hostptr->h_addr + 3 );
    hsnd.sender = addr;

    ssock = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );
    if ( ssock == -1 ) {
        printf( "\n\r\x1b[1;31mERROR:\x1b[m Can't initialize socket...\n\n\r" );
        exit( 1 );
    }else
	printf( "Send Socket: OK\n" );

    buf_size = 1;
    setsockopt( ssock, SOL_SOCKET, SO_BROADCAST, (char *)&buf_size, sizeof(buf_size) );
    buf_size = BUF;
    setsockopt( ssock, SOL_SOCKET, SO_SNDBUF, (char *)&buf_size, sizeof(buf_size) );

    toaddr.sin_family = AF_INET;
    toaddr.sin_port = htons( INPORT );
    toaddr.sin_addr.s_addr = INADDR_BROADCAST;

    rsock = socket( PF_INET, SOCK_DGRAM, 0 );
    if ( rsock == -1 ) {
        printf( "\n\r\x1b[1;31mERROR:\x1b[m Can't initialize socket...\n\n\r" );
        exit( 1 );
    }else
	 printf( "Receive Socket: OK\n" );

    buf_size = 1;
    setsockopt( rsock, SOL_SOCKET, SO_BROADCAST, &buf_size, sizeof( buf_size ) );
    buf_size = BUF;
    setsockopt( rsock, SOL_SOCKET, SO_RCVBUF, &buf_size, sizeof( buf_size ) );

    outaddr.sin_family = AF_INET;
    outaddr.sin_port = htons( INPORT );
    outaddr.sin_addr.s_addr = htonl( INADDR_ANY );

    if ( bind( rsock, (struct sockaddr *)(&outaddr), sizeof(struct sockaddr) ) < 0 ) {
                printf( "\n\r\x1b[1;31mERROR:\x1b[m Can't bind receive socket. Perheps you're \n\r" );
                printf( "trying to run lanchat twice on the same machine.\n\n" );
                exit( 1 );
            } 

    if ( ( cp2 = fork() ) == -1 ) {
        endwin();
        perror( "fork" );
        exit( 1 );
    }

    if ( cp2 == 0 ) { /* we are in cild process */
        while ( 1 ) {
            hsnd.dest = hsnd.sender;
            hsnd.type = LC_LINUX_REQUEST;
            snd();
            sleep( 5 );
        }
    }
/*    else 	*/  /* ???*/

    /* let's rock with interface */
    initscr();
	
    headwin   = newwin( 1,       COLS,    0,       0 );
    defwin    = newwin( LINES-4, COLS-20, 1,       0 );
    userwin   = newwin( LINES-4, 16,      1,       COLS-17 );
    split2win = newwin( LINES-4, 1,       1,       COLS-19 );
    split1win = newwin( 1,       COLS,    LINES-3, 0 );
    inputwin  = newwin( 1,       COLS,    LINES-2, 0 );
    split3win = newwin( 1,       COLS,    LINES-1, 0 );
			    
    scrollok( defwin, TRUE );
    scrollok( inputwin, TRUE );
    keypad( inputwin, TRUE );
    nonl();
		
    if (has_colors()){
        start_color();
        init_pair( 1, COLOR_WHITE, COLOR_BLUE );
        init_pair( 5, COLOR_WHITE, COLOR_BLACK );
        init_pair( 4, COLOR_CYAN, COLOR_BLACK );
        init_pair( 2, COLOR_BLUE, COLOR_BLACK );
        init_pair( 3, COLOR_MAGENTA, COLOR_BLACK );
        init_pair( 6, COLOR_GREEN, COLOR_BLACK );
        init_pair( 7, COLOR_RED, COLOR_BLACK );
        init_pair( 8, COLOR_YELLOW, COLOR_BLACK );
    }

    werase( headwin );
    werase( defwin );
    werase( userwin );
    werase( inputwin );
    werase( split1win );
    werase( split2win );
    werase( split3win );

    wattron( headwin, COLOR_PAIR( 1 ) );
    wbkgdset( headwin, COLOR_PAIR( 1 ) );

    wattron( split1win, COLOR_PAIR( 1 ) | A_BOLD );
    wbkgdset( split1win, COLOR_PAIR( 1 ) );

    wattron( split2win, COLOR_PAIR( 2 ) );
    wattron( split3win, COLOR_PAIR( 2 ) );

    /* vertical line */
    for ( i = 0; i < LINES-2; i++ ) mvwaddch( split2win, i, 0, ACS_VLINE );
    /* the lines at the bottom */
    for ( i = 0; i < COLS; i++ ) waddch( split1win, ' ' );
    for ( i = 0; i < COLS; i++ ) mvwaddch( split3win, 0, i, ACS_HLINE );
    
    updateall();

    finish = FALSE;
    option = FALSE;

    signal( SIGINT, use_exit );

    strcpy( since, atime( 0 ) );
    strcpy( topic, "<>" );

    strcpy( hsnd.nick, logname ); /* logname is safe */
    pop_mode = 0; /* allow popup msgs */

    wattrset( defwin, COLOR_PAIR( 4 ) );
    wprintw( defwin, "Welcome to " );
    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
    wprintw( defwin, "Linux LANChat v%s\n\n", VER );
//    wattrset( defwin, COLOR_PAIR( 4 ) );
//    wprintw( defwin, " Melun melun@friko2.onet.pl\n" );
//    wprintw( defwin, "Distributed without any warranty under GPL version 2 terms\n" );
//    wprintw( defwin, "and conditions.\n" );
//    wprintw( defwin, "This program is a Linux adaptation of the original LANChat\n" );
//    wprintw( defwin, "for windows. Type " );
//    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
//    wprintw( defwin, "/help " );
//    wattrset( defwin, COLOR_PAIR( 4 ) );
//    wprintw( defwin, "for commands summary.\n" );
//    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
//    wprintw( defwin, "http://lanchat.topnet.pl\n\n" );
    wattrset( defwin, COLOR_PAIR( 5 ) );

    if ( ( cp = fork() ) == -1 ) {
        endwin();
        perror( "fork" );
        exit( 1 );
    }

    if ( cp == 0 ) { /* we are in child process */
        while ( 1 ) {
            buf_size = sizeof( struct sockaddr );
            recv = recvfrom( rsock, &hrcv, BUF, 0,
		    (struct sockaddr *)&oth, (unsigned int *)&buf_size );

            if ( recv != -1 ) {
                bytes_rcv += recv;
                total_msg++;
            }
            /* simple packet checking */
            if ( ( hrcv.cHead[ 0 ] != 'L' ) || ( hrcv.cHead[ 1 ] != 'C' )
                    || ( hrcv.cHead[ 2 ] != '4' ) || ( hrcv.cHead[ 3 ] != '0' ) ) {
                wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
                wprintw( defwin, "DANGER: Strange packet was received on lanchat port.\n" );
                wattrset( defwin, COLOR_PAIR( 5 ) );

                updatewin( defwin );
            }
            else if(users[ hrcv.sender ].lag == 3)
		show_resurrected();
            if ( hrcv.dest == hsnd.sender || hrcv.dest == LC_BROADCAST )
                switch ( hrcv.type ) {

                /*** internal messages ***/
		case LC_LINUX_NULL:
		    break;
                case LC_LINUX_MSG:   /* send msg */
                    if ( !prepare_message() ) {
                        wattrset( defwin, COLOR_PAIR( 7 ) );
                        wprintw( defwin, "msg" );
                        wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
                        wprintw( defwin, " --> " );
                        wattrset( defwin, COLOR_PAIR( 4 ) | A_NORMAL );
                        wprintw( defwin, "(" );
                        wattrset( defwin, COLOR_PAIR( 5 ) );
                        wprintw( defwin, "%s", users[ hsnd.dest ].nick );
                        wattrset( defwin, COLOR_PAIR( 4 ) );
                        wprintw( defwin, ")   " );
                        wattrset( defwin, COLOR_PAIR( 5 ) );
                        hsnd.data[ strlen( hsnd.data ) - 2 ] = '\0';
                        wprintw( defwin, "%s\n", hsnd.data );
                        
			updatewin( defwin );

                        hsnd.type = LC_NORMAL;
                        strcat( hsnd.data, "\x0d\x0a\0" );
                        crypt( hsnd.data, hsnd.sender, hsnd.dest );
                        snd();
                    }
                    break;
                case LC_LINUX_STATUS:
                    show_stats();
                    break;
                case LC_LINUX_POP:
                    if ( !prepare_message() ) {
                        hsnd.type = LC_POPUP;
                        crypt( hsnd.data, hsnd.sender, hsnd.dest );
                        snd();
                    }
                    break;
                case LC_LINUX_HELP:
                    show_help();
                    break;
                case LC_LINUX_CREDITS:
                    show_credits();
                    break;
                case LC_LINUX_UNKCMD:
                    show_unknown();
                    break;
                case LC_LINUX_WHOIS:
                    decr();
                    if ( auser( hrcv.data ) != -1 ) {
                        hsnd.type = LC_REQ_STATUS_EX;
                        hsnd.dest = auser( hrcv.data );
                        snd();
                    }
                    else {
                        wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
                        wprintw( defwin, "No such user: " );
                        wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
                        wprintw( defwin, "%s\n", hrcv.data );
                        wattrset( defwin, COLOR_PAIR( 5 ) );
			updatewin( defwin );
                    }
                    break;
                case LC_LINUX_NICKLN:
                    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
                    wprintw( defwin, "Nick length have to be >2 and <20.\n" );
                    wattrset( defwin, COLOR_PAIR( 5 ) );
                    updatewin( defwin );
                    break;
		case LC_LINUX_CLEAR:
                    werase(defwin);
		    wnoutrefresh(userwin);
		    make_userlist();
                    updatewin( defwin );
		    break;												
                case LC_LINUX_REQUEST:
                    hsnd.dest = LC_BROADCAST;
                    if ( users[ hsnd.sender ].away == 0 ) hsnd.type = LC_ALIVE;
                    else hsnd.type = LC_AWAY_ALIVE;
                    snd();
                    for ( i = 0; i < 255; i++ ) 
                        if ( strlen( users[ i ].nick ) > 0 && i != hsnd.sender ) {
                            users[ i ].lag++;
                            if ( users[ i ].lag == 3 ) {
                                wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
                                wprintw( defwin, "%s ", users[ i ].nick );
                                wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
                                wprintw( defwin, "timed out at " );
                                wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
                                wprintw( defwin, "%s", atime( 0 ) );
                                wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
                                wprintw( defwin, ".\n" );
                                wattrset( defwin, COLOR_PAIR( 5 ) );

                                users[ i ].nick[ 0 ] = '\0';
                                make_userlist();
                                updatewin( defwin );
                            }
                        }
                    break;

                /*** external messagess ***/
                case LC_ADD_TO_ROOM:
                    users[ hrcv.sender ].room = 1;
                    break;
                case LC_REMOVE_FROM_ROOM:
                    users[ hrcv.sender ].room = 0;
                    break;
                case LC_TOPIC:
                case LC_NEWTOPIC:
                    change_topic();
                    break;
                case LC_TOPIC_TIME:
                    decrypt( hrcv.xVal, hrcv.data, hrcv.sender, hrcv.dest );
                    hrcv.data[ strlen( hrcv.data ) - 2 ] = '\0';
                    strncpy( topictime, hrcv.data, 8 );
                    break;
                case LC_POPOK:
                case LC_CANTPOP:
                    if ( hrcv.type == LC_POPOK )
                        wprintw( defwin, "POP-UP has been received.\n", hrcv.nick );
                    else
                        wprintw( defwin, "%s has popup's disabled.\n", hrcv.nick );
                    wattrset( defwin, COLOR_PAIR( 5 ) );
                    updatewin( defwin );
                    break;
                case LC_POPUP:
//                    if ( hrcv.sender != hsnd.sender ) {
                        if ( pop_mode == 0 ) {
                            hsnd.type = LC_POPOK;
                            show_pop();
                        }else
			    hsnd.type = LC_CANTPOP;
                    	hsnd.dest = hrcv.sender;
                    	snd();
//                    }
                    break;
                case LC_MOTHER:
                    wattrset( defwin, COLOR_PAIR( 5 ) | A_BOLD );
                    wprintw( defwin, "%s has a mother attack!\n", hrcv.nick );
                    wattrset( defwin, COLOR_PAIR( 5 ) | A_NORMAL );
                    if ( hrcv.sender == hsnd.sender ) {
		        werase( defwin );
			updatewin( defwin );
		    }
		    break;
                case LC_REQ_STATUS_EX:
                    hsnd.dest = hrcv.sender;
                    hsnd.type = LC_STATUS_EX;

                    snprintf( hsnd.data, 4092,
                             "%s\x07%s@%s\x07%s\x07LANChat %s\x07%s\x07%d\x07%s\x07",
			     hsnd.nick, logname, host, myip, VER, OS,
                             users[ hsnd.sender ].away, since );
		    hsnd.data[4092]='\0';
                    strcat( hsnd.data, "\x0d\x0a\0" );
                    crypt( hsnd.data, hsnd.sender, hsnd.dest );
                    snd();
                    break;
                case LC_STATUS_EX:
                    if ( hrcv.dest == hsnd.sender )
			show_status();
                    break;
                case LC_NORMAL:
                    if ( hrcv.dest == LC_BROADCAST )
                        show_normal();
                    else
                        show_msg();
                    break;
                case LC_ME:
                    show_me();
                    break;
                case LC_JOIN:
                    show_join();
                    strncpy( users[ hrcv.sender ].nick, hrcv.nick, 19 );
		    users[ hrcv.sender ].nick[19]='\0';
                    users[ hrcv.sender ].away = 0;
                    make_userlist();
                    hsnd.dest = hrcv.sender;
                    if( users[ hsnd.sender ].away == 0 )
			hsnd.type = LC_ALIVE;
                    else
			hsnd.type = LC_AWAY_ALIVE;
                    snd();
                    hsnd.type = LC_TOPIC;
                    strncpy( hsnd.data, topic, 4092 ); hsnd.data[4092]='\0';
                    strcat( hsnd.data, "\x0d\x0a\0" );
                    crypt( hsnd.data, hsnd.sender, hsnd.dest );
                    snd();
                    hsnd.type = LC_TOPIC_TIME;
                    strncpy( hsnd.data, topictime, 4092 ); hsnd.data[4092]='\0';
                    strcat( hsnd.data, "\x0d\x0a\0" );
                    crypt( hsnd.data, hsnd.sender, hsnd.dest );
                    snd();
                    break;
                case LC_LEAVE:
                    users[ hrcv.sender ].nick[ 0 ] = '\0';
                    show_leave();
                    make_userlist();
                    if ( hrcv.sender == hsnd.sender ) exit( 0 );
                    break;
                case LC_AWAY:
                    show_away();
                    break;
                case LC_ALIVE:
                    strncpy( users[ hrcv.sender ].nick, hrcv.nick, 19 );
		    users[ hrcv.sender ].nick[19]='\0';
                    users[ hrcv.sender ].away = 0;
                    make_userlist();
                    users[ hrcv.sender ].lag = 0;
                    break;
                case LC_REQ_AWAY:
                    if ( users[ hsnd.sender ].away == 1 ) {
                        hsnd.dest = hrcv.sender;
                        hsnd.type = LC_AWAY;
                        strncpy( hsnd.data, away, 4092 ); hsnd.data[4092]='\0';
                        strcat( hsnd.data, "\x0d\x0a\0" );
                        crypt( hsnd.data, hsnd.sender, hsnd.dest );
                        snd();
                    }
                    break;
                case LC_REQ_ALIVE:
                    hsnd.dest = hrcv.sender;
                    if ( users[ hsnd.sender ].away == 0 )
			hsnd.type = LC_ALIVE;
                    else
			hsnd.type = LC_AWAY_ALIVE;
                    snd();
                    break;
                case LC_START_AWAY:
                    users[ hrcv.sender ].away = 1;
                    show_went_away();
                    make_userlist();
                    break;
                case LC_END_AWAY:
                    users[ hrcv.sender ].away = 0;
                    show_back_away();
                    make_userlist();
                    break;
                case LC_AWAY_ALIVE:
                    strncpy( users[ hrcv.sender ].nick, hrcv.nick, 19 );
		    users[ hrcv.sender ].nick[19]='\0';
                    users[ hrcv.sender ].away = 1;
                    make_userlist();
                    users[ hrcv.sender ].lag = 0;
                    break;
                case LC_NEWNICK:
                    if ( hrcv.sender == hsnd.sender ){
			strncpy( hsnd.nick, hrcv.nick, 19 );
			hsnd.nick[19]='\0';
		    };
                    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
                    wprintw( defwin, "*  " );
                    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
                    wprintw( defwin, "%s", users[ hrcv.sender ].nick );
                    wattrset( defwin, COLOR_PAIR( 2 ) | A_BOLD );
                    wprintw( defwin, " is now known as " );
                    wattrset( defwin, COLOR_PAIR( 4 ) | A_BOLD );
                    wprintw( defwin, "%s\n", hrcv.nick );
                    wattrset( defwin, COLOR_PAIR( 5 ) );

                    updatewin( defwin );

                    strncpy( users[ hrcv.sender ].nick, hrcv.nick, 19 );
		    users[ hrcv.sender ].nick[19]='\0';
                    make_userlist();
                    break;
                }
        }
    } /* end of child */
    else {
        flog( "LANChat Session opened at: " );
        flog( atime( 0 ) );
        flog( "\n\n" );
        hsnd.type = LC_JOIN;
        hsnd.dest = LC_BROADCAST;
        snprintf( hsnd.data, 4092, "(%s, IP=%s)", host, myip );
	hsnd.data[4092]='\0';
        strcat( hsnd.data, "\x0d\x0a\0" );
        crypt( hsnd.data, hsnd.sender, hsnd.dest );
        snd();

        while ( !finish ) {
	    mvwgetnstr( inputwin, 0, 0, readbuffer, 4000); /* reading the line */
	    strncpy(hsnd.data, readbuffer, 4000);
	    hsnd.data[4000]='\0';
            option = FALSE;
            hsnd.type = 0;
            hsnd.dest = hsnd.sender;

            if ( !cmp( hsnd.data, "/quit" ) ) {
                finish = TRUE;
                hsnd.dest = LC_BROADCAST;
                hsnd.type = LC_LEAVE;
                hsnd.data[ 0 ] = '\0';
            }
            if ( !cmp( hsnd.data, "/whois " ) ) {
                option = TRUE;
                hsnd.dest = hsnd.sender;
                hsnd.type = LC_LINUX_WHOIS;
                strncpy( hsnd.data, hsnd.data + 7, strlen( hsnd.data ) - 7 );
                hsnd.data[ strlen( hsnd.data ) - 7 ] = '\0';
                strcat( hsnd.data, "\x0d\x0a\0" );

            }
            if ( !cmp( hsnd.data, "/away " ) ) {
                option = TRUE;
                hsnd.dest = LC_BROADCAST;
                hsnd.type = LC_START_AWAY;
                strncpy( hsnd.data, hsnd.data + 6, strlen( hsnd.data ) - 6 );
                hsnd.data[ strlen( hsnd.data ) - 6 ] = '\0';
                strcat( hsnd.data, "\x0d\x0a\0" );
                away_mode = 1;
            }
            if ( !cmp( hsnd.data, "/back" ) ) {
                option = TRUE;
        	if ( away_mode == 1 ) {
	            hsnd.type = LC_END_AWAY;
		    hsnd.dest = LC_BROADCAST;
		    away_mode = 0;
		}else{
            	    hsnd.dest = hsnd.sender;
		    hsnd.type = LC_LINUX_NULL;
		    hsnd.data[ 0 ] = '\0';
		};
            }
            if ( !cmp( hsnd.data, "/topic " ) ) {
                option = TRUE;
                hsnd.dest = LC_BROADCAST;
                hsnd.type = LC_NEWTOPIC;
                strncpy( hsnd.data, hsnd.data + 7, strlen( hsnd.data ) - 7 );
                hsnd.data[ strlen( hsnd.data ) - 7 ] = '\0';
                strcat( hsnd.data, "\x0d\x0a\0" );
                crypt( hsnd.data, hsnd.sender, hsnd.dest );
                snd();
                hsnd.type = LC_TOPIC_TIME;
                strcpy( hsnd.data, atime( 0 ) );
                strcat( hsnd.data, "\x0d\x0a\0" );
            }
            if ( !cmp( hsnd.data, "/sa" ) ) {
                option = TRUE;
                hsnd.dest = LC_BROADCAST;
                hsnd.type = LC_REQ_AWAY;
                hsnd.data[ 0 ] = '\0';
            }
            if ( !cmp( hsnd.data, "/status" ) ) {
                option = TRUE;
                hsnd.dest = hsnd.sender;
                hsnd.type = LC_LINUX_STATUS;
                hsnd.data[ 0 ] = '\0';
            }
            if ( !cmp( hsnd.data, "/help" ) ) {
                option = TRUE;
                hsnd.dest = hsnd.sender;
                hsnd.type = LC_LINUX_HELP;
                hsnd.data[ 0 ] = '\0';
            }
            if ( !cmp( hsnd.data, "/credits" ) ) {
                option = TRUE;
                hsnd.dest = hsnd.sender;
                hsnd.type = LC_LINUX_CREDITS;
                hsnd.data[ 0 ] = '\0';
            }
            if ( !cmp( hsnd.data, "/msg " )) {
                option = TRUE;
                hsnd.dest = hsnd.sender;
                hsnd.type = LC_LINUX_MSG;
                strncpy( hsnd.data, hsnd.data + 5, strlen( hsnd.data ) - 5 );
                hsnd.data[ strlen( hsnd.data ) - 5 ] = '\0';
                strcat( hsnd.data, "\x0d\x0a\0" );
            }
            if ( !cmp( hsnd.data, "/pop " ) ) {
                option = TRUE;
                hsnd.dest = hsnd.sender;
                hsnd.type = LC_LINUX_POP;
                strncpy( hsnd.data, hsnd.data + 5, strlen( hsnd.data ) - 5 );
                hsnd.data[ strlen( hsnd.data ) - 5 ] = '\0';
                strcat( hsnd.data, "\x0d\x0a\0" );
            }
            if ( !cmp( hsnd.data, "/clear" ) || !cmp( hsnd.data, "/cls" )) {
                option = TRUE;
                hsnd.dest = hsnd.sender;
                hsnd.type = LC_LINUX_CLEAR;
                hsnd.data[ 0 ] = '\0';
            }
	    if( !cmp(hsnd.data, "/mother")){
		option = TRUE;
		hsnd.dest = LC_BROADCAST;
		hsnd.type = LC_MOTHER;
		hsnd.data[ 0 ] = '\0';
	    }
            if ( !cmp( hsnd.data, "/me " ) ) {
                option = TRUE;
                hsnd.dest = LC_BROADCAST;
                hsnd.type = LC_ME;
                strncpy( hsnd.data, hsnd.data + 4, strlen( hsnd.data ) - 4 );
                hsnd.data[ strlen( hsnd.data ) - 4 ] = '\0';
                strcat( hsnd.data, "\x0d\x0a\0" );
            }
            if ( !cmp( hsnd.data, "/nick " ) ) {
                option = TRUE;
                hsnd.dest = LC_BROADCAST;
                hsnd.type = LC_NEWNICK;
                strncpy( hsnd.data, hsnd.data + 6, strlen( hsnd.data ) - 6 );
                hsnd.data[ strlen( hsnd.data ) - 6 ] = '\0';
                if ( ( strlen( hsnd.data ) < 3 ) || ( strlen( hsnd.data ) > 19 ) )
                    hsnd.type = LC_LINUX_NICKLN;
                else {
                    strncpy( users[ hsnd.sender ].nick, hsnd.data, 19 );
                    users[ hsnd.sender ].nick[ 19 ] = '\0';
                    strncpy( hsnd.nick, users[ hsnd.sender ].nick, 19 );
		    hsnd.nick[19]='\0';
                }
            }

            if ( ( strlen( hsnd.data ) > 0 ) && ( !finish ) && ( !option ) ) {
		if(away_mode==1){ /* termination of away mode */
                    hsnd.type = LC_END_AWAY;
            	    hsnd.dest = LC_BROADCAST;
		    snd();
		    away_mode = 0;
		}								
                strcat( hsnd.data, "\x0d\x0a\0" );
                hsnd.type = LC_NORMAL;
                hsnd.dest = LC_BROADCAST;
		if(away_mode==1)
		    away_mode=2; /* force termination of away mode */
            }

            if ( ( !option ) && !cmp( hsnd.data, "/" ) ) {
                hsnd.type = LC_LINUX_UNKCMD;
                hsnd.dest = hsnd.sender;
            }
            crypt( hsnd.data, hsnd.sender, hsnd.dest );
            snd();
            wdeleteln( inputwin );
            wrefresh( inputwin );
        }
    }

    flog( "\nLANChat Session closed at: " );
    flog( atime( 0 ) );
    flog( "\n\n" );
    flog( "===================================\n\n" );
    cleanup();
    kill( cp2, SIGTERM );
    return( 0 );
}

/* end lc_main.c */
